require "util"
require "defines"
require "story"

-- localisation shortcut
gt = game.gettext

game.oninit(function()
  game.player.force.disableallprototypes()
  game.mapsettings.enemy_expansion.enabled = false
  game.evolutionfactor = 0.45
  game.mapsettings.enemy_evolution.enabled = false

  local recipelist = game.player.force.recipes
  recipelist["iron-plate"].enabled = true
  recipelist["copper-plate"].enabled = true
  recipelist["stone-brick"].enabled = true
  recipelist["wall"].enabled = true
  recipelist["wood"].enabled = true
  recipelist["stone-furnace"].enabled = true
  recipelist["iron-stick"].enabled = true
  recipelist["iron-axe"].enabled = true
  recipelist["wooden-chest"].enabled = true
  recipelist["iron-gear-wheel"].enabled = true
  recipelist["basic-transport-belt"].enabled = true
  recipelist["electronic-circuit"].enabled = true
  recipelist["copper-cable"].enabled = true
  recipelist["pistol"].enabled = true
  recipelist["submachine-gun"].enabled = true
  recipelist["basic-bullet-magazine"].enabled = true
  recipelist["basic-mining-drill"].enabled = true
  recipelist["basic-inserter"].enabled = true
  recipelist["small-electric-pole"].enabled = true
  recipelist["solar-panel"].enabled = true
  recipelist["basic-accumulator"].enabled = true
  recipelist["repair-pack"].enabled = true

  local technologylist = game.player.force.technologies
  technologylist["steel-processing"].researched = true
  technologylist["optics"].researched = true
  technologylist["logistics"].researched = true
  technologylist["logistics-2"].researched = true
  technologylist["automation"].researched = true
  technologylist["automation-2"].researched = true
  technologylist["flame-thrower"].enabled = true
  technologylist["electronics"].researched = true
  technologylist["advanced-electronics"].researched = true
  technologylist["military"].researched = true
  technologylist["military-2"].researched = true
  technologylist["turrets"].researched = true
  technologylist["armor-making"].researched = true
  technologylist["bullet-damage-1"].researched = true
  technologylist["bullet-damage-2"].researched = true
  technologylist["bullet-speed-1"].researched = true
  technologylist["bullet-speed-2"].researched = true
  technologylist["toolbelt"].researched = true

  local character = game.player.character
  character.insert{name = "fast-transport-belt", count = 128}
  character.insert{name = "fast-inserter", count = 45}
  character.insert{name = "medium-electric-pole", count = 40}
  character.insert{name = "long-handed-inserter", count = 30}
  character.insert{name = "assembling-machine-2", count = 20}
  character.insert{name = "wall", count = 128}
  character.insert{name = "gun-turret", count = 15}
  character.insert{name = "steel-chest", count = 5}
  character.insert{name = "coal", count = 200}
  character.insert{name = "submachine-gun", count = 1}
  character.insert{name = "piercing-bullet-magazine", count = 100}
  character.insert{name = "steel-axe", count = 1}
  character.insert{name = "heavy-armor", count = 1}
  character.insert{name = "steel-plate", count = 1500}
  character.insert{name = "solar-panel", count = 50}
  character.insert{name = "basic-accumulator", count = 50}
  character.insert{name = "iron-plate", count = 500}
  character.insert{name = "copper-plate", count = 500}

  local car = game.getentitybytag("car")
  car.insert{name = "piercing-bullet-magazine", count = 2500}
  car.insert{name = "iron-plate", count = 2500}
  car.insert{name = "copper-plate", count = 2500}
  car.insert{name = "coal", count = 57}
  
  local techwreckage = game.getentitybytag("tech-wreckage")
  techwreckage.insert{name = "rocket-launcher", count = 1}
  techwreckage.insert{name = "rocket", count = 15}
  techwreckage.insert{name = "defender-capsule", count = 20}
  techwreckage.insert{name = "slowdown-capsule", count = 10}
  
  local bonuswreckage = game.getentitybytag("bonus-wreckage")
  bonuswreckage.insert{name = "defender-capsule", count = 10}
  bonuswreckage.insert{name = "rocket", count = 10}
end)

storytable =
{
  {
    {
      condition = story.Story:elapsedcheck(2),
      action = 
      function()
        game.showmessagedialog(gt("msg-wreck-location"))
        game.showmessagedialog(gt("msg-wreck-motivation"))
        game.showmessagedialog(gt("msg-wreck-guarded"))
      end
    },
    {
      init =
      function(event)
        game.player.setgoaldescription(gt("goal-get-wreckage"))
      end,
    },
    {
      condition =
      function()
        local wreck = game.getentitybytag("tech-wreckage")
        if wreck ~= nil then
          return game.getentitybytag("tech-wreckage").equals(game.player.opened)
        else
          game.showmessagedialog(gt("msg-wreck-destroyed"))
          return true
        end
      end
    },
    {
      condition = story.Story:elapsedcheck(5),
      init =
      function(event)
        game.player.setgoaldescription("")
        game.showmessagedialog(gt("msg-could-make-more-1"))
        game.showmessagedialog(gt("msg-could-make-more-2"))
        local technologies = game.player.force.technologies
        technologies["rocketry"].researched = true
        technologies["military-3"].researched = true
        technologies["combat-robotics"].researched = true
        technologies["follower-robot-count-1"].researched = true
        technologies["follower-robot-count-2"].researched = true
        technologies["follower-robot-count-3"].researched = true
        technologies["follower-robot-count-4"].researched = true
        technologies["follower-robot-count-5"].researched = true
        technologies["follower-robot-count-6"].researched = true
        technologies["follower-robot-count-7"].researched = true
        technologies["follower-robot-count-8"].researched = true
      end
    },
    {
      condition = story.Story:elapsedcheck(5),
      init =
      function(event)
        game.player.setgoaldescription(gt("goal-get-computer"))
      end
    },
    { condition = function() return game.player.getitemcount("computer") > 0 end }
  }
}

mainstory = story.Story:new(storytable)

game.onsave(function()
  glob.storydata = mainstory:export()
end)

game.onload(function()
  mainstory:import(glob.storydata)
end)

function difficultynumber(easy, normal, hard)
  if game.difficulty == defines.difficulty.easy then
    return easy
  elseif game.difficulty == defines.difficulty.normal then
    return normal
  else
    return hard
  end
end

function manageattacks(tick)
  if glob.lastattackat == nil then
    glob.lastattackat = difficultynumber(120, 60, 0)
  end
  if glob.attackcount == nil then
    glob.attackcount = 3
  end
  -- attack rate is once per 90 seconds, it doesn't change
  if glob.attackfrequency == nil then
    glob.attackfrequency = difficultynumber(50, 40, 30)
  end
  if tick - glob.lastattackat > 60 * glob.attackfrequency then
    glob.lastattackat = tick
    game.setmulticommand({type=defines.command.attack,
                          target=game.player.character,
                          distraction=defines.distraction.byenemy},
                          glob.attackcount)
    -- every attack is 0.6 aliens bigger
    glob.attackcount = glob.attackcount + 0.7
  end
end

game.onevent(defines.events, function(event)
  manageattacks(event.tick)
  mainstory:update(event, "level-04")
end)
